@@ -16,19 +16,9 @@
 
 // All the things we need from Imath.
 #define OSL_USING_IMATH 3
-#if OSL_USING_IMATH >= 3
-#   include <Imath/ImathVec.h>
-#   include <Imath/ImathMatrix.h>
-#   include <Imath/ImathColor.h>
-#else
-    // OpenEXR 2.x lacks the Cuda decorators we need, so we replicated some
-    // Imath files in OSL/Imathx, adding the decorations needed for them to
-    // be safe to compile with Cuda (and sometimes other improvements).
-#   include <OSL/Imathx/ImathLimits.h>
-#   include <OSL/Imathx/ImathVec.h>
-#   include <OSL/Imathx/ImathMatrix.h>
-#   include <OSL/Imathx/ImathColor.h>
-#endif
+#include <Imath/ImathVec.h>
+#include <Imath/ImathMatrix.h>
+#include <Imath/ImathColor.h>
 
 // We included the Imath files we needed, so set the OIIO_IMATH_H_INCLUDED
 // symbol to prevent OpenImageIO/Imath.h from including them again (or
@@ -50,7 +40,7 @@
 #    ifdef OIIO_ERRORHANDLER_HIDE_PRINTF
 #        define OIIO_ERRORHANDLER_HIDE_PRINTF 1
 #    endif
-#    ifdef OIIO_HIDE_FORMAT
+#    ifndef OIIO_HIDE_FORMAT
 #        define OIIO_HIDE_FORMAT 1
 #    endif
 #endif
@@ -65,7 +55,7 @@
 #include <OpenImageIO/span.h>
 
 
-OSL_NAMESPACE_ENTER
+OSL_NAMESPACE_BEGIN
 
 /// Various compile-time defaults are defined here that could, in
 /// principle, be redefined if you are using OSL in some particular
@@ -79,7 +69,7 @@ OSL_NAMESPACE_ENTER
 typedef float Float;
 
 /// By default, use the excellent Imath vector, matrix, and color types
-/// from the IlmBase package from: http://www.openexr.com
+/// from https://github.com/AcademySoftwareFoundation/Imath
 ///
 /// It's permissible to override these types with the vector, matrix,
 /// and color classes of your choice, provided that (a) your vectors
@@ -90,6 +80,7 @@ typedef float Float;
 using Vec2     = Imath::Vec2<Float>;
 using Vec3     = Imath::Vec3<Float>;
 using Color3   = Imath::Color3<Float>;
+using Matrix22 = Imath::Matrix22<Float>;
 using Matrix33 = Imath::Matrix33<Float>;
 using Matrix44 = Imath::Matrix44<Float>;
 
@@ -151,31 +142,21 @@ fmtformat_to_n(OutIt &out, size_t n, const string_view& fmt, Args&&... args)
 }
 
 
-/// OSL_USTRINGREP_IS_HASH will be 1 if the build-time option was enabled to
-/// make the ustringrep be a ustringhash, rather than a ustring directly.
-#define OSL_USTRINGREP_IS_HASH 0
-
-#if OSL_USTRINGREP_IS_HASH
-/// ustringrep is the class we use to represent strings in the shader.
-using ustringrep = ustringhash;
-#else
-/// ustringrep is the class we use to represent strings in the shader.
-using ustringrep = ustring;
-#endif
-
 /// ustringhash_pod is the type we use to pass hash data in llvm function calls.
 using ustringhash_pod = size_t;
 
-#if OSL_USTRINGREP_IS_HASH
-/// ustring_pod is the type we use to pass string data in llvm function calls.
-using ustring_pod = ustringhash_pod;
-#else
 /// ustring_pod is the type we use to pass string data in llvm function calls.
 using ustring_pod = const char*;
-#endif
 
 
 
+/// Convenience function to convert to a ustring.
+inline ustring
+ustring_from(ustringhash_pod h)
+{
+    return ustring::from_hash(h);
+}
+
 /// Convenience function to convert to a ustring.
 inline ustring
 ustring_from(ustringhash h)
@@ -190,23 +171,16 @@ ustring_from(ustring u)
     return u;
 }
 
-/// Convenience function to convert to a ustring.
-inline ustring
-ustring_from(string_view s)
-{
-    return ustring(s);
-}
-
 
 /// Convenience function to convert to a ustringhash.
-inline ustringhash
+OSL_HOSTDEVICE inline ustringhash
 ustringhash_from(ustringhash u)
 {
     return u;
 }
 
 /// Convenience function to convert USTRINGHASH_POD to a ustringhash.
-inline ustringhash
+OSL_HOSTDEVICE inline ustringhash
 ustringhash_from(ustringhash_pod u) 
 {
 #if OIIO_VERSION_GREATER_EQUAL(2, 4, 10)
@@ -230,49 +204,13 @@ ustringhash_from(ustring u)
 }
 
 
-/// Convenience function to convert to a ustringrep.
-inline ustringrep
-ustringrep_from(ustringhash h)
-{
-#if OSL_USTRINGREP_IS_HASH
-    return h;
-#else
-    return ustring::from_hash(h.hash());
-#endif
-}
-
-/// Convenience function to convert to a ustringrep.
-inline ustringrep
-ustringrep_from(ustring u)
-{
-#if OSL_USTRINGREP_IS_HASH
-    return u.hash();
-#else
-    return u;
-#endif
-}
-
-/// Convenience function to convert to a ustringrep.
-inline ustringrep
-ustringrep_from(string_view s)
-{
-    ustring u(s);
-#if OSL_USTRINGREP_IS_HASH
-    return u.hash();
-#else
-    return u;
-#endif
-}
-
-
-
 using TypeDesc_pod = int64_t;
 
 static_assert(sizeof(TypeDesc_pod) == sizeof(TypeDesc),
               "TypeDesc size differs from its POD counterpart");
 
 /// Convenience function to convert to a TypeDesc.
-inline TypeDesc
+OSL_HOSTDEVICE inline TypeDesc
 TypeDesc_from(TypeDesc_pod type)
 {
     return OSL::bitcast<OSL::TypeDesc>(type);
@@ -280,6 +218,19 @@ TypeDesc_from(TypeDesc_pod type)
 
 
 
+/// Options for the trace call.
+struct TraceOpt {
+    float mindist;         ///< ignore hits closer than this
+    float maxdist;         ///< ignore hits farther than this
+    bool shade;            ///< whether to shade what is hit
+    ustringhash traceset;  ///< named trace set
+    OSL_HOSTDEVICE TraceOpt() : mindist(0.0f), maxdist(1.0e30), shade(false) {}
+
+    enum class LLVMMemberIndex { mindist = 0, maxdist, shade, traceset, count };
+};
+
+
+
 // N.B. SymArena is not really "configuration", but we cram it here for
 // lack of a better home.
 
@@ -301,19 +252,8 @@ enum class SymArena {
 // various C++ versions or compilers, or to make up for things not
 // yet available in earlier C++ standards.
 
-#if OSL_CPLUSPLUS_VERSION >= 17
-    // fold expression to expand
-    #define __OSL_EXPAND_PARAMETER_PACKS(EXPRESSION) \
-        (void((EXPRESSION)) , ... );
-#else
-    // Choose to use initializer list to expand our parameter pack so that
-    // the order they are evaluated in the order they appear
-    #define __OSL_EXPAND_PARAMETER_PACKS(EXPRESSION) \
-        { \
-            using expander = int [sizeof...(IntListT)]; \
-            (void)expander{((EXPRESSION),0)...}; \
-        }
-#endif
+// fold expression to expand
+#define __OSL_EXPAND_PARAMETER_PACKS(EXPRESSION) (void((EXPRESSION)), ...);
 
 
 namespace pvt {
@@ -324,26 +264,20 @@ using int_sequence = std::integer_sequence<int, IntegerListT...>;
 template<int EndBeforeT>
 using make_int_sequence =  std::make_integer_sequence<int, EndBeforeT>;
 
- template<bool... BoolListT >
-using bool_sequence = std::integer_sequence<bool, BoolListT... >;
+template<bool... BoolListT>
+using bool_sequence = std::integer_sequence<bool, BoolListT...>;
 
-#if (OSL_CPLUSPLUS_VERSION >= 17)
-    template<class... ListT>
-    using conjunction = std::conjunction<ListT...>;
-#else
-    template<class... ListT>
-    using conjunction = std::is_same<bool_sequence<true,ListT::value...>, bool_sequence<ListT::value..., true>>;
-#endif
+template<class... ListT> using conjunction = std::conjunction<ListT...>;
 
-    // We need the SFINAE type to be different for
-    // enable_if_type from disable_if_type so that we can apply both to
-    // the same template signature to avoid
-    // "error: invalid redeclaration of member function template"
-    // NOTE: std::enable_if_t is a c++14 library feature, our baseline
-    // and we wish to remain compatible with c++11 header libraries.
-    // Also we are using std::true_type vs. void as the default type
-    template <bool TestT, typename TypeT = std::true_type>
-    using enable_if_type = typename std::enable_if<TestT, TypeT>::type;
+// We need the SFINAE type to be different for
+// enable_if_type from disable_if_type so that we can apply both to
+// the same template signature to avoid
+// "error: invalid redeclaration of member function template"
+// NOTE: std::enable_if_t is a c++14 library feature, our baseline
+// and we wish to remain compatible with c++11 header libraries.
+// Also we are using std::true_type vs. void as the default type
+template<bool TestT, typename TypeT = std::true_type>
+using enable_if_type = typename std::enable_if<TestT, TypeT>::type;
 
 } // namespace pvt
 
@@ -372,7 +306,7 @@ using ConstIndex = std::integral_constant<int, N>;
 #endif
 
 
-OSL_NAMESPACE_EXIT
+OSL_NAMESPACE_END
 
 
 namespace std {  // not necessary in C++17, then we can just say std::hash
